#!/bin/bash
#
# buildRoot.sh <gcc47|e2|mu2e:gcc47|nu:gcc47|mu2e:e2|nu:e2> <opt|debug|prof>

basequal=${1}
extraqual=${2}
pkgver=v5_34_01
pkgdotver=`echo ${pkgver} | sed -e 's/_/./g' | sed -e 's/^v//'`
tarfile=root_v${pkgdotver}.source.tar.gz
patchfile=fermi_root_5_34.patch
gcctestver=4.7

# check to see if qualifier is set
if [ -z ${basequal} ]
then
   echo "ERROR: please define the qualifier"
   echo "USAGE: buildRoot.sh <gcc47|e2|mu2e:gcc47|nu:gcc47|mu2e:e2|nu:e2> <debug|opt|prof>"
   exit 1
fi

if [ "${basequal}" = "gcc47" ] 
then
    shortqual=gcc
    qdir=${basequal}
elif [ "${basequal}" = "e2" ] 
then
    shortqual=e
    cqual=e2
    qdir=${basequal}
    c11="--enable-c++11"
elif [ "${basequal}" = "nu:gcc47" ] 
then
    shortqual=nu
    qdir=nu-gcc47
elif [ "${basequal}" = "mu2e:gcc47" ] 
then
    shortqual=mu2e
    qdir=mu2e-gcc47
elif [ "${basequal}" = "nu:e2" ] 
then
    shortqual=nu
    cqual=e2
    qdir=nu-e2
    c11="--enable-c++11"
elif [ "${basequal}" = "mu2e:e2" ] 
then
    shortqual=mu2e
    cqual=e2
    qdir=mu2e-e2
    c11="--enable-c++11"
else
   echo "ERROR: unrecognized qualifier"
   echo "USAGE: buildRoot.sh <gcc47|e2|mu2e:gcc47|nu:gcc47|mu2e:e2|nu:e2> <debug|opt|prof>"
   exit 1
fi

# use ROOTSYS/MyRules.mk
if [ "${extraqual}" = "opt" ]
then
    fullqual=${basequal}:${extraqual}
    qualdir=${qdir}-${extraqual}
    rules=MyRules_opt.mk
elif [ "${extraqual}" = "debug" ]
then
    fullqual=${basequal}:${extraqual}
    qualdir=${qdir}-${extraqual}
    extracommand="--build=debug"
    rules=MyRules_debug.mk
elif [ "${extraqual}" = "prof" ]
then
    fullqual=${basequal}:${extraqual}
    qualdir=${qdir}-${extraqual}
    rules=MyRules_prof.mk
else
    # default CXXFLAGS are -g -O2
   echo "ERROR: please specify debug, opt, or prof"
   echo "USAGE: buildRoot.sh <gcc47|e2|mu2e:gcc47|nu:gcc47|mu2e:e2|nu:e2> <debug|opt|prof>"
   exit 1
##    fullqual=${basequal}
##    qualdir=${qdir}
##    extracommand="--build=debug"
fi

echo "compiling for ${fullqual}"

source ${PRODUCTS}/setup
OS=`uname`
if [ ${OS} = "Darwin" ]
then
    flvr=`ups flavor -2`
else
    flvr=`ups flavor`
fi
rootdir=${PRODUCTS}/root/${pkgver}/${flvr}-${qualdir}
sourcedir=${PRODUCTS}/root/${pkgver}/source

if [ -z ${flvr} ]
then
   echo "ERROR: could not determine flavor for this platform"
   echo "       something is very wrong"
   exit 1
fi

# declare now so we can setup
mkdir -p ${rootdir}
if [ ! -d ${rootdir} ]
then
   echo "ERROR: failed to create ${rootdir}"
   exit 1
fi

echo "Declaring root ${pkgver} ${flvr} -q ${fullqual}"
if [ ${OS} = "Darwin" ]
then
   ups declare root ${pkgver} -r root/${pkgver} -2 -m root.table -q ${fullqual}
else 
   ups declare root ${pkgver} -r root/${pkgver} -4 -m root.table -q ${fullqual}
fi

# setup root
setup root ${pkgver} -q ${fullqual}
gccver=`gcc --version | head -1 | cut -f3 -d" " | cut -f1-2 -d"."`
gccfullver=`gcc --version | head -1 | cut -f3 -d" "`
if [ "${gccver}" = "${gcctestver}" ]
then
  echo " OK - gcc version is ${gccfullver}"
else
  echo "ERROR: wrong gcc version ${gccfullver}"
  exit 1
fi

if [ -z ${FFTW_INC} ]
then
   echo "ERROR: failed to setup fftw"
   exit 1
else
   echo "FFTW_INC: ${FFTW_INC}"
fi

if [ -z ${LIBXML2_INC} ]
then
   echo "ERROR: failed to setup libxml"
   exit 1
else
   echo "LIBXML2_INC: ${LIBXML2_INC}"
fi

if [ "${shortqual}" = "nu" ] 
then
    if [ -z ${POSTGRESQL_VERSION} ]
    then
       echo "ERROR: failed to setup postgresql"
       exit 1
    else
       echo "postgresql version: ${POSTGRESQL_VERSION}"
    fi

    if [ -z ${MYSQL_CLIENT_VERSION} ]
    then
       echo "ERROR: failed to setup mysql_client"
       exit 1
    else
       echo "mysql_client version: ${MYSQL_CLIENT_VERSION}"
    fi
fi

if [ -z ${PYTHONHOME} ]
then
   echo "ERROR: PYTHONHOME is undefined"
   echo "       something is very wrong"
   exit 1
fi

cd ${rootdir}
tar xzf ${sourcedir}/${tarfile}

# get the extra rules 
if [ -n ${rules} ]
then
   cp ${ROOT_DIR}/${rules} ${rootdir}/root/MyRules.mk
fi
if [ "${cqual}" = "e2" ] 
then
  echo "CXXFLAGS += -std=c++11 -Wno-deprecated-declarations" >> ${rootdir}/root/MyRules.mk
  echo "NOOPT += -std=c++11 -Wno-deprecated-declarations" >> ${rootdir}/root/MyRules.mk
fi

set -x

cd ${rootdir}/root

# apply Fermi patches
patch -b -p0 < ${sourcedir}/${patchfile}

if [ "${shortqual}" = "nu" ]
then
./configure ${extracommand} \
--enable-gdml ${c11} \
--with-fftw3-incdir=${FFTW_INC} \
--with-fftw3-libdir=${FFTW_FQ_DIR}/lib/ \
--disable-xrootd \
--enable-minuit2 \
--disable-xft \
--with-xml-incdir=${LIBXML2_INC} \
--with-xml-libdir=${LIBXML2_FQ_DIR}/lib \
--with-pythia6-libdir=${PYLIB} \
--with-pgsql-incdir=${POSTGRESQL_FQ_DIR}/include \
--with-pgsql-libdir=${POSTGRESQL_FQ_DIR}/lib \
--with-mysql-incdir=${MYSQL_CLIENT_FQ_DIR}/include \
--with-mysql-libdir=${MYSQL_CLIENT_FQ_DIR}/lib
elif [ "${shortqual}" = "mu2e" ]
then
./configure ${extracommand} \
--enable-gdml ${c11} \
--enable-roofit \
--with-fftw3-incdir=${FFTW_INC} \
--with-fftw3-libdir=${FFTW_FQ_DIR}/lib/ \
--disable-mysql \
--disable-xrootd \
--enable-minuit2 \
--disable-xft \
--with-xml-incdir=${LIBXML2_INC} \
--with-xml-libdir=${LIBXML2_FQ_DIR}/lib
else
./configure ${extracommand} \
--enable-gdml ${c11} \
--with-fftw3-incdir=${FFTW_INC} \
--with-fftw3-libdir=${FFTW_FQ_DIR}/lib/ \
--disable-mysql \
--disable-xrootd \
--enable-minuit2 \
--disable-xft \
--with-xml-incdir=${LIBXML2_INC} \
--with-xml-libdir=${LIBXML2_FQ_DIR}/lib
fi

make -j3
# now install in ROOTSYS
make install 

set +x

if [ ! -d ${rootdir}/lib ]
then
   echo "ERROR: failed to create ${rootdir}/lib"
   echo "       Something is very wrong"
   exit 1
fi

# this should not complain
echo "Finished building root ${pkgver} ${flvr} -q ${fullqual}"
setup root ${pkgver} -q ${fullqual}
echo "root is installed at ${ROOTSYS}"

exit 0
